@@ -3,6 +3,7 @@
  * This file is part of GtkSourceView
  *
  * Copyright (C) 2009 - Jesse van den Kieboom <jessevdk@gnome.org>
+ * Copyright (C) 2016 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,10 +42,10 @@ struct _GtkSourceCompletionItemPrivate
 	gchar *label;
 	gchar *markup;
 	gchar *text;
-	gchar *info;
 	GdkPixbuf *icon;
 	gchar *icon_name;
 	GIcon *gicon;
+	gchar *info;
 };
 
 enum
@@ -69,52 +70,52 @@ G_DEFINE_TYPE_WITH_CODE (GtkSourceCompletionItem,
 			 			gtk_source_completion_proposal_iface_init))
 
 static gchar *
-gtk_source_completion_proposal_get_label_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_label_impl (GtkSourceCompletionProposal *proposal)
 {
-	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (self)->priv->label);
+	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->label);
 }
 
 static gchar *
-gtk_source_completion_proposal_get_markup_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_markup_impl (GtkSourceCompletionProposal *proposal)
 {
-	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (self)->priv->markup);
+	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->markup);
 }
 
 static gchar *
-gtk_source_completion_proposal_get_text_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_text_impl (GtkSourceCompletionProposal *proposal)
 {
-	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (self)->priv->text);
+	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->text);
 }
 
 static GdkPixbuf *
-gtk_source_completion_proposal_get_icon_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_icon_impl (GtkSourceCompletionProposal *proposal)
 {
-	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->icon;
+	return GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->icon;
 }
 
 static const gchar *
-gtk_source_completion_proposal_get_icon_name_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_icon_name_impl (GtkSourceCompletionProposal *proposal)
 {
-	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->icon_name;
+	return GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->icon_name;
 }
 
 static GIcon *
-gtk_source_completion_proposal_get_gicon_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_gicon_impl (GtkSourceCompletionProposal *proposal)
 {
-	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->gicon;
+	return GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->gicon;
 }
 
 static gchar *
-gtk_source_completion_proposal_get_info_impl (GtkSourceCompletionProposal *self)
+gtk_source_completion_proposal_get_info_impl (GtkSourceCompletionProposal *proposal)
 {
-	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (self)->priv->info);
+	return g_strdup (GTK_SOURCE_COMPLETION_ITEM (proposal)->priv->info);
 }
 
 static void
 gtk_source_completion_proposal_iface_init (gpointer g_iface,
 					   gpointer iface_data)
 {
-	GtkSourceCompletionProposalIface *iface = (GtkSourceCompletionProposalIface *)g_iface;
+	GtkSourceCompletionProposalIface *iface = g_iface;
 
 	/* Interface data getter implementations */
 	iface->get_label = gtk_source_completion_proposal_get_label_impl;
@@ -126,28 +127,27 @@ gtk_source_completion_proposal_iface_init (gpointer g_iface,
 	iface->get_info = gtk_source_completion_proposal_get_info_impl;
 }
 
+static void
+gtk_source_completion_item_dispose (GObject *object)
+{
+	GtkSourceCompletionItem *item = GTK_SOURCE_COMPLETION_ITEM (object);
+
+	g_clear_object (&item->priv->icon);
+	g_clear_object (&item->priv->gicon);
+
+	G_OBJECT_CLASS (gtk_source_completion_item_parent_class)->dispose (object);
+}
+
 static void
 gtk_source_completion_item_finalize (GObject *object)
 {
-	GtkSourceCompletionItem *self = GTK_SOURCE_COMPLETION_ITEM(object);
+	GtkSourceCompletionItem *item = GTK_SOURCE_COMPLETION_ITEM (object);
 
-	g_free (self->priv->label);
-	g_free (self->priv->markup);
-	g_free (self->priv->text);
-
-	g_free (self->priv->info);
-
-	if (self->priv->icon != NULL)
-	{
-		g_object_unref (self->priv->icon);
-	}
-
-	g_free (self->priv->icon_name);
-
-	if (self->priv->gicon != NULL)
-	{
-		g_object_unref (self->priv->gicon);
-	}
+	g_free (item->priv->label);
+	g_free (item->priv->markup);
+	g_free (item->priv->text);
+	g_free (item->priv->icon_name);
+	g_free (item->priv->info);
 
 	G_OBJECT_CLASS (gtk_source_completion_item_parent_class)->finalize (object);
 }
@@ -158,35 +158,42 @@ gtk_source_completion_item_get_property (GObject    *object,
 					 GValue     *value,
 					 GParamSpec *pspec)
 {
-	GtkSourceCompletionItem *self;
+	GtkSourceCompletionItem *item;
 
 	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (object));
 
-	self = GTK_SOURCE_COMPLETION_ITEM (object);
+	item = GTK_SOURCE_COMPLETION_ITEM (object);
 
 	switch (prop_id)
 	{
 		case PROP_LABEL:
-			g_value_set_string (value, self->priv->label);
+			g_value_set_string (value, item->priv->label);
 			break;
+
 		case PROP_MARKUP:
-			g_value_set_string (value, self->priv->markup);
+			g_value_set_string (value, item->priv->markup);
 			break;
+
 		case PROP_TEXT:
-			g_value_set_string (value, self->priv->text);
-			break;
-		case PROP_INFO:
-			g_value_set_string (value, self->priv->info);
+			g_value_set_string (value, item->priv->text);
 			break;
+
 		case PROP_ICON:
-			g_value_set_object (value, self->priv->icon);
+			g_value_set_object (value, item->priv->icon);
 			break;
+
 		case PROP_ICON_NAME:
-			g_value_set_string (value, self->priv->icon_name);
+			g_value_set_string (value, item->priv->icon_name);
 			break;
+
 		case PROP_GICON:
-			g_value_set_object (value, self->priv->gicon);
+			g_value_set_object (value, item->priv->gicon);
 			break;
+
+		case PROP_INFO:
+			g_value_set_string (value, item->priv->info);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -205,56 +212,42 @@ gtk_source_completion_item_set_property (GObject      *object,
 					 const GValue *value,
 					 GParamSpec   *pspec)
 {
-	GtkSourceCompletionItem *self;
+	GtkSourceCompletionItem *item;
 
 	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (object));
 
-	self = GTK_SOURCE_COMPLETION_ITEM (object);
+	item = GTK_SOURCE_COMPLETION_ITEM (object);
 
 	switch (prop_id)
 	{
 		case PROP_LABEL:
-			g_free (self->priv->label);
-			self->priv->label = g_value_dup_string (value);
-			emit_changed (self);
+			gtk_source_completion_item_set_label (item, g_value_get_string (value));
 			break;
+
 		case PROP_MARKUP:
-			g_free (self->priv->markup);
-			self->priv->markup = g_value_dup_string (value);
-			emit_changed (self);
+			gtk_source_completion_item_set_markup (item, g_value_get_string (value));
 			break;
+
 		case PROP_TEXT:
-			g_free (self->priv->text);
-			self->priv->text = g_value_dup_string (value);
-			break;
-		case PROP_INFO:
-			g_free (self->priv->info);
-			self->priv->info = g_value_dup_string (value);
-			emit_changed (self);
+			gtk_source_completion_item_set_text (item, g_value_get_string (value));
 			break;
+
 		case PROP_ICON:
-			if (self->priv->icon != NULL)
-			{
-				g_object_unref (self->priv->icon);
-			}
-
-			self->priv->icon = GDK_PIXBUF (g_value_dup_object (value));
-			emit_changed (self);
+			gtk_source_completion_item_set_icon (item, g_value_get_object (value));
 			break;
+
 		case PROP_ICON_NAME:
-			g_free (self->priv->icon_name);
-			self->priv->icon_name = g_value_dup_string (value);
-			emit_changed (self);
+			gtk_source_completion_item_set_icon_name (item, g_value_get_string (value));
 			break;
-		case PROP_GICON:
-			if (self->priv->gicon != NULL)
-			{
-				g_object_unref (self->priv->gicon);
-			}
 
-			self->priv->gicon = G_ICON (g_value_dup_object (value));
-			emit_changed (self);
+		case PROP_GICON:
+			gtk_source_completion_item_set_gicon (item, g_value_get_object (value));
 			break;
+
+		case PROP_INFO:
+			gtk_source_completion_item_set_info (item, g_value_get_string (value));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -266,6 +259,7 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = gtk_source_completion_item_dispose;
 	object_class->finalize = gtk_source_completion_item_finalize;
 	object_class->get_property = gtk_source_completion_item_get_property;
 	object_class->set_property = gtk_source_completion_item_set_property;
@@ -279,9 +273,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_LABEL,
 					 g_param_spec_string ("label",
 							      "Label",
-							      "Label to be shown for this item",
+							      "",
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceCompletionItem:markup:
@@ -292,9 +287,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_MARKUP,
 					 g_param_spec_string ("markup",
 							      "Markup",
-							      "Markup to be shown for this item",
+							      "",
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceCompletionItem:text:
@@ -305,9 +301,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_TEXT,
 					 g_param_spec_string ("text",
 							      "Text",
-							      "Item text",
+							      "",
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceCompletionItem:icon:
@@ -318,9 +315,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_ICON,
 					 g_param_spec_object ("icon",
 							      "Icon",
-							      "Pixbuf of the icon to be shown for this item",
+							      "",
 							      GDK_TYPE_PIXBUF,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceCompletionItem:icon-name:
@@ -333,9 +331,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_ICON_NAME,
 					 g_param_spec_string ("icon-name",
 							      "Icon Name",
-							      "Icon name of the icon to be shown for this item",
+							      "",
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GtkSourceCompletionItem:gicon:
@@ -348,9 +347,11 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_GICON,
 					 g_param_spec_object ("gicon",
 							      "GIcon",
-							      "GIcon of the icon to be shown for this item",
+							      "",
 							      G_TYPE_ICON,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * GtkSourceCompletionItem:info:
 	 *
@@ -360,15 +361,16 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 					 PROP_INFO,
 					 g_param_spec_string ("info",
 							      "Info",
-							      "Info to be shown for this item",
+							      "",
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 }
 
 static void
-gtk_source_completion_item_init (GtkSourceCompletionItem *self)
+gtk_source_completion_item_init (GtkSourceCompletionItem *item)
 {
-	self->priv = gtk_source_completion_item_get_instance_private (self);
+	item->priv = gtk_source_completion_item_get_instance_private (item);
 }
 
 /**
@@ -383,6 +385,7 @@ gtk_source_completion_item_init (GtkSourceCompletionItem *self)
  * there will be no icon shown and no extra information available.
  *
  * Returns: a new #GtkSourceCompletionItem.
+ * Deprecated: 3.24: Use gtk_source_completion_item_new2() instead.
  */
 GtkSourceCompletionItem *
 gtk_source_completion_item_new (const gchar *label,
@@ -410,6 +413,7 @@ gtk_source_completion_item_new (const gchar *label,
  * which case there will be no icon shown and no extra information available.
  *
  * Returns: a new #GtkSourceCompletionItem.
+ * Deprecated: 3.24: Use gtk_source_completion_item_new2() instead.
  */
 GtkSourceCompletionItem *
 gtk_source_completion_item_new_with_markup (const gchar *markup,
@@ -438,7 +442,7 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
  * the stock label will be used.
  *
  * Returns: a new #GtkSourceCompletionItem.
- * Deprecated: 3.10: Use gtk_source_completion_item_new() instead.
+ * Deprecated: 3.10: Use gtk_source_completion_item_new2() instead.
  */
 GtkSourceCompletionItem *
 gtk_source_completion_item_new_from_stock (const gchar *label,
@@ -486,3 +490,175 @@ gtk_source_completion_item_new_from_stock (const gchar *label,
 }
 
 G_GNUC_END_IGNORE_DEPRECATIONS;
+
+/**
+ * gtk_source_completion_item_new2:
+ *
+ * Creates a new #GtkSourceCompletionItem. The desired properties need to be set
+ * afterwards.
+ *
+ * Returns: (transfer full): a new #GtkSourceCompletionItem.
+ * Since: 3.24
+ */
+GtkSourceCompletionItem *
+gtk_source_completion_item_new2 (void)
+{
+	return g_object_new (GTK_SOURCE_TYPE_COMPLETION_ITEM, NULL);
+}
+
+/**
+ * gtk_source_completion_item_set_label:
+ * @item: a #GtkSourceCompletionItem.
+ * @label: (nullable): the label, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_label (GtkSourceCompletionItem *item,
+				      const gchar             *label)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+
+	if (g_strcmp0 (item->priv->label, label) != 0)
+	{
+		g_free (item->priv->label);
+		item->priv->label = g_strdup (label);
+
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "label");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_markup:
+ * @item: a #GtkSourceCompletionItem.
+ * @markup: (nullable): the markup, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_markup (GtkSourceCompletionItem *item,
+				       const gchar             *markup)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+
+	if (g_strcmp0 (item->priv->markup, markup) != 0)
+	{
+		g_free (item->priv->markup);
+		item->priv->markup = g_strdup (markup);
+
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "markup");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_text:
+ * @item: a #GtkSourceCompletionItem.
+ * @text: (nullable): the text, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_text (GtkSourceCompletionItem *item,
+				     const gchar             *text)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+
+	if (g_strcmp0 (item->priv->text, text) != 0)
+	{
+		g_free (item->priv->text);
+		item->priv->text = g_strdup (text);
+
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "text");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_icon:
+ * @item: a #GtkSourceCompletionItem.
+ * @icon: (nullable): the #GdkPixbuf, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_icon (GtkSourceCompletionItem *item,
+				     GdkPixbuf               *icon)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+	g_return_if_fail (icon == NULL || GDK_IS_PIXBUF (icon));
+
+	if (g_set_object (&item->priv->icon, icon))
+	{
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "icon");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_icon_name:
+ * @item: a #GtkSourceCompletionItem.
+ * @icon_name: (nullable): the icon name, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_icon_name (GtkSourceCompletionItem *item,
+					  const gchar             *icon_name)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+
+	if (g_strcmp0 (item->priv->icon_name, icon_name) != 0)
+	{
+		g_free (item->priv->icon_name);
+		item->priv->icon_name = g_strdup (icon_name);
+
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "icon-name");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_gicon:
+ * @item: a #GtkSourceCompletionItem.
+ * @gicon: (nullable): the #GIcon, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_gicon (GtkSourceCompletionItem *item,
+				      GIcon                   *gicon)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+	g_return_if_fail (gicon == NULL || G_IS_ICON (gicon));
+
+	if (g_set_object (&item->priv->gicon, gicon))
+	{
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "gicon");
+	}
+}
+
+/**
+ * gtk_source_completion_item_set_info:
+ * @item: a #GtkSourceCompletionItem.
+ * @info: (nullable): the info, or %NULL.
+ *
+ * Since: 3.24
+ */
+void
+gtk_source_completion_item_set_info (GtkSourceCompletionItem *item,
+				     const gchar             *info)
+{
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_ITEM (item));
+
+	if (g_strcmp0 (item->priv->info, info) != 0)
+	{
+		g_free (item->priv->info);
+		item->priv->info = g_strdup (info);
+
+		emit_changed (item);
+		g_object_notify (G_OBJECT (item), "info");
+	}
+}
