@@ -31,18 +31,8 @@
 
 #define COMPOSITE_ALPHA                 225
 
-struct _GtkSourceGutterRendererMarksPrivate
-{
-};
-
 G_DEFINE_TYPE (GtkSourceGutterRendererMarks, gtk_source_gutter_renderer_marks, GTK_SOURCE_TYPE_GUTTER_RENDERER_PIXBUF)
 
-static void
-gtk_source_gutter_renderer_marks_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (gtk_source_gutter_renderer_marks_parent_class)->finalize (object);
-}
-
 static gint
 sort_marks_by_priority (gconstpointer m1,
 			gconstpointer m2,
@@ -400,11 +390,8 @@ gutter_renderer_change_view (GtkSourceGutterRenderer *renderer,
 static void
 gtk_source_gutter_renderer_marks_class_init (GtkSourceGutterRendererMarksClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkSourceGutterRendererClass *renderer_class = GTK_SOURCE_GUTTER_RENDERER_CLASS (klass);
 
-	object_class->finalize = gtk_source_gutter_renderer_marks_finalize;
-
 	renderer_class->query_data = gutter_renderer_query_data;
 	renderer_class->query_tooltip = gutter_renderer_query_tooltip;
 	renderer_class->query_activatable = gutter_renderer_query_activatable;
